# -*- coding: UTF-8 -*-
# Public package
import time
import pandas
import inspect
# Private package
# Internal package

################################################################################
# 批注
global_buffer = pandas.DataFrame(columns=['Module', 'Function', 'Time']).set_index(['Module', 'Function'])['Time']
################################################################################


def dec(func):  # 修饰器，记录函数运行时间
    def wrapper(*args, **argv):
        start = time.time()
        result = func(*args, **argv)
        end = time.time()
        module = inspect.getmodule(func).__name__
        function = func.__qualname__
        if ((module, function) not in global_buffer):
            global_buffer[(module, function)] = end - start
        else:
            global_buffer[(module, function)] += end - start
        return result
    return wrapper


def __buffer_to_df(sort=False):
    output = pandas.DataFrame({'time(s)': global_buffer,
                               'percent(%)': global_buffer / global_buffer.sum() * 100})
    if (sort):
        output = output.sort_values(by='time(s)', ascending=False)
    else:
        output = output.sort_index()
    return output


def report(logger=None, sort=False):
    output = __buffer_to_df(sort=sort)
    if (logger is not None):
        for line in output.__str__().split('\n'):
            logger.info(line)
    else:
        print(output)


def to_csv(filename):
    output = __buffer_to_df()
    output.to_csv(filename)
    return output


def to_pickle(filename):
    output = __buffer_to_df()
    output.to_pickle(filename)
    return output



# -*- coding: UTF-8 -*-
# Public package
import os
import time
import logging
import inspect
import colorlog
# Private package
# Internal package


def log(name: str = '',  # log名称
        stream: bool = True,  # 是否输出屏幕
        file: str = None,  # 输出文件
        level: str = 'INFO') -> logging.Logger:  # 等级
    logger = logging.getLogger(name=name)
    logger.handlers.clear()
    logger.setLevel(level)
    handlers = []
    if (stream):
        handler = logging.StreamHandler()
        formatter = colorlog.ColoredFormatter('[%(red)s%(asctime)s%(reset)s][%(blue)s%(name)s%(reset)s][%(log_color)s%(levelname)s%(reset)s]: %(message)s')
        handler.setFormatter(formatter)
        handlers.append(handler)
    if (file is not None):
        if (os.path.dirname(file) != '' and not os.path.exists(os.path.dirname(file))):
            os.makedirs(os.path.dirname(file), exist_ok=True)
        handler = logging.FileHandler(file, 'w+')
        formatter = logging.Formatter('[%(asctime)s][%(name)s][%(levelname)s]: %(message)s')
        handler.setFormatter(formatter)
        handlers.append(handler)
    for handler in handlers:
        handler.setLevel(level)
        logger.addHandler(handler)
    logger.propagate = False
    return logger


def get_class_logger(object, **argv) -> logging.Logger:  # 返回某一个类的自动logger
    return log(name='%s.%s' % (inspect.getmodule(object).__name__, type(object).__qualname__), **argv)


def get_func_logger(func, **argv) -> logging.Logger:  # 返回某一个函数的自动logger
    return log(name='%s.%s' % (inspect.getmodule(func).__name__, func.__qualname__), **argv)


def dec_timer(func):  # 修饰器，返回函数运行时间
    def output_func(*args, **argv):
        time_start = time.perf_counter()
        result = func(*args, **argv)
        time_end = time.perf_counter()
        logger = get_func_logger(func, level='INFO')
        logger.info('time cost: {:.2f}s'.format(time_end - time_start))
        return result
    return output_func
